                 (* +-----------------------------+
                    |  Files et suite de Hamming  |
                    +-----------------------------+ *)

type 'a file = {mutable dbut : 'a list;
                mutable fin   : 'a list};;

let new_file() = {dbut = []; fin = []};;
let push  f x = f.dbut <- x :: f.dbut;;
let queue f x = f.fin   <- x :: f.fin;;

let pop f = match f.dbut with
| x::suite -> f.dbut <- suite; x
| []       -> match rev(f.fin) with
              | x::suite -> f.dbut <- suite; f.fin <- []; x
              | []       -> failwith "file vide"
;;
let peek f = let x = pop f in push f x; x;;
let remove f = let _ = pop f in ();;
let lg f = list_length(f.dbut) + list_length(f.fin);;


let h2 = new_file();;
let h3 = new_file();;
let h5 = new_file();;

let ln2 = log(2.0)
and ln3 = log(3.0)
and ln5 = log(5.0);;
 
let minimum (a,b,c as x) (a',b',c' as y) =
  let t = float_of_int(a-a')*.ln2
       +. float_of_int(b-b')*.ln3
       +. float_of_int(c-c')*.ln5 in
  if t <= 0. then x else y
;;

let hnext() =
  let x2 = peek(h2)
  and x3 = peek(h3)
  and x5 = peek(h5) in
  let x  = minimum (minimum x2 x3) x5 in
  if x=x2 then remove h2;
  if x=x3 then remove h3;
  if x=x5 then remove h5;
  let (a,b,c) = x in
  queue h2 (a+1,b,c);
  if a = 0 then begin
    queue h3 (a,b+1,c);
    if b = 0 then queue h5 (a,b,c+1)
  end;
  x
;;

let hamming(n) =
  h2.dbut <- []; h2.fin <- [0,0,0];
  h3.dbut <- []; h3.fin <- [0,0,0];
  h5.dbut <- []; h5.fin <- [0,0,0];
  for i=1 to n do let _ = hnext() in () done;
  hnext()
;;


